part of 'exercise_control_bloc.dart';

@immutable
abstract class ExerciseControlState extends Equatable {
  const ExerciseControlState();

  @override
  List<Object> get props => [];
}

class ExerciseControlInitial extends ExerciseControlState {
  const ExerciseControlInitial();
}

class ExerciseControlLoading extends ExerciseControlState {
  const ExerciseControlLoading();
}

class ExerciseControlReady extends ExerciseControlState {
  const ExerciseControlReady();
}

class ExerciseControlError extends ExerciseControlState {
  final String message;
  const ExerciseControlError({required this.message});

  @override
  List<Object> get props => [message];
}
